#include "EquationData.h"

EquationData::EquationData() : _p(DBL_MIN), _cg(DBL_MIN), _c(DBL_MIN), _dt(DBL_MIN), _h(DBL_MIN), _k(DBL_MIN), _metric(DBL_MAX){}

EquationData::EquationData(double p, double cg, double c, double dt, double h, double k) : _metric(DBL_MAX){
	setP(p);
	setCg(cg);
	setC(c);
	setDt(dt);
	setH(h);
	setK(k);
}

void EquationData::setP(double p){
	if (p < EquationBounds::pmin){
		_p = EquationBounds::pmin;
		_p_out = true;
	}
	else if (p > EquationBounds::pmax){
		_p = EquationBounds::pmax;
		_p_out = true;
	}
	else {
		_p = p;
	}
}
void EquationData::setCg(double cg){
	if (cg < EquationBounds::cgmin){
		_cg = EquationBounds::cgmin;
		_cg_out = true;
	}
	else if (cg > EquationBounds::cgmax){
		_cg = EquationBounds::cgmax;
		_cg_out = true;
	}
	else {
		_cg = cg;
	}
}
void EquationData::setC(double c){
	if (c < EquationBounds::cmin){
		_c = EquationBounds::cmin;
		_c_out = true;
	}
	else if (c > EquationBounds::cmax){
		_c = EquationBounds::cmax;
		_c_out = true;
	}
	else {
		_c = c;
	}
}
void EquationData::setDt(double dt){
	if (dt < EquationBounds::dtmin){
		_dt = EquationBounds::dtmin;
		_dt_out = true;
	}
	else if (dt > EquationBounds::dtmax){
		_dt = EquationBounds::dtmax;
		_dt_out = true;
	}
	else {
		_dt = dt;
	}
}
void EquationData::setH(double h){
	if (h < EquationBounds::hmin){
		_h = EquationBounds::hmin;
		_h_out = true;
	}
	else if (h > EquationBounds::hmax){
		_h = EquationBounds::hmax;
		_h_out = true;
	}
	else if (h == 0.0) {
		_h += DBL_EPSILON;
	}
	else {
		_h = h;
	}
}
void EquationData::setK(double k){
	if (k < EquationBounds::kmin){
		_k = EquationBounds::kmin;
		_k_out = true;
	}else if(k > EquationBounds::kmax){
		_k = EquationBounds::kmax;
		_k_out = true;
	}
	else {
		_k = k;
	}
}

void EquationData::setMetric(double metric){
	_metric = metric;
}

void EquationData::clear(){
	_p = 0;
	_cg = 0;
	_c = 0;
	_dt = 0;
	_h = 0;
	_k = 0;
}

void EquationData::setAllParameters(double p, double cg, double c, double dt, double h, double k){
	setP(p);
	setCg(cg);
	setC(c);
	setDt(dt);
	setH(h);
	setK(k);
}

std::ostream& operator<<(std::ostream &strm, const EquationData &x) {
	return strm << "EquationData("
		<< "P = " << x.getP() << "; "
		<< "CG = " << x.getCg() << "; "
		<< "C = " << x.getC() << "; "
		<< "DT = " << x.getDt() << "; "
		<< "H = " << x.getH() << "; "
		<< "K = " << x.getK() << "; "
		<< "metric = " << x.getMetric() << ")";
}

bool operator<(EquationData const & L, EquationData const & R) {
	return L.getMetric() < R.getMetric();
}