#include "SqliteDriver.h"

SqliteDriver::SqliteDriver(string path){
	if (!ifstream(path) || sqlite3_open(path.c_str(), &_db) != SQLITE_OK) {
		throw "SQLite file cannot be opened: " + path + " -> " + string(sqlite3_errmsg(_db));
	}
	//cout << "SQLite file was opened" << endl;
}

SqliteDriver::~SqliteDriver(void){
	sqlite3_close(_db);
}


vector<SegmentData> SqliteDriver::load(vector<SegmentData> &segments){
	string sql_query, sql_date_subquery;
	sqlite3_stmt *statement;
	int sql_row = 0, segment_id = INT_MIN, local_segment_id = INT_MIN;
	vector<MeasuredData> segment_values;


	sql_date_subquery = "(strftime('%s', datetime(measuredat)) + 2209161600) * (1.0 / 86400.0)";
	sql_query = "SELECT id, blood, ist, segmentid, " + sql_date_subquery + " AS date";
	sql_query += " FROM measuredvalue";
	sql_query += " ORDER BY segmentid, measuredat";

	if (sqlite3_prepare_v2(_db, sql_query.c_str(), -1, &statement, 0) != SQLITE_OK) {
		sql_err_code = sqlite3_errcode(_db);
		ostringstream error;
		error << "SQLite error (code " << sql_err_code << "): " << sqlite3_errmsg(_db);
		error << "\n" << sql_query << endl;
		throw error.str();
	}
	else {
		while (true){
			sql_row = sqlite3_step(statement);
			if (sql_row == SQLITE_ROW) {
				MeasuredData row_data;

				row_data.setRowId(sqlite3_column_int(statement, 0));
				row_data.setBlood(sqlite3_column_double(statement, 1)); //blood column
				row_data.setIst(sqlite3_column_double(statement, 2)); //ist column
				row_data.setDate(sqlite3_column_double(statement, 4)); //date column
				local_segment_id = sqlite3_column_int(statement, 3); //segmentid column

				// Prvotni nastaveni segment_id
				if (segment_id == INT_MIN) {
					//cout << "First segment: " << local_segment_id << endl;
					segment_id = local_segment_id;
				}

				if (segment_id != local_segment_id){
					//cout << "New segment: " << local_segment_id << endl;
					SegmentData sg(local_segment_id, segment_values);
					segments.push_back(sg);
					segment_values.clear();
					segment_id = local_segment_id;
				}
				segment_values.push_back(row_data);
			}
			else {
				//cout << "Last segment: " << segment_id << endl;
				SegmentData sg(local_segment_id, segment_values);
				segments.push_back(sg);
				break;
			}
		}
		sqlite3_finalize(statement);
	}
	return segments;
}

void SqliteDriver::save(vector<SegmentData> &segments){
	string sql_query;
	ostringstream values_string;
	char *sql_errmsg = 0;

	int segment_id = -1;

	for (int size = segments.size(), i = 0; i < size; i++) {
		values_string.str("");
		values_string.clear();
		values_string << segments[i].getResult().getP() << ","
			<< segments[i].getResult().getCg() << ","
			<< segments[i].getResult().getC() << ","
			<< segments[i].getResult().getDt() << ","
			<< segments[i].getResult().getH() << ","
			<< segments[i].getResult().getK() << ","
			<< "1,"
			<< segments[i].getId();
		segment_id = segments[i].getId();
		sql_query = "INSERT INTO difuse2params (p, cg, c, dt, h, k, s, segmentid) VALUES (" + values_string.str() + ");";

		if (sqlite3_exec(_db, sql_query.c_str(), NULL, 0, &sql_errmsg) != SQLITE_OK){
			string err = string(sql_errmsg);
			sqlite3_free(sql_errmsg);
			sql_err_code = sqlite3_errcode(_db);
			ostringstream error;
			error << "SQLite error (code " << sql_err_code << "): " << err;
			error << "\n" << sql_query << endl;
			throw error.str();
		}
		//else {
		//	cout << "Saving segment " << segment_id << " was OK" << endl;
		//}
	}

}