#include "SegmentData.h"


SegmentData::SegmentData() 
	: _id(0), _number_min_metric(NUMBER_MIN_METRIC), _values(vector<MeasuredData>()), _blood_values(vector<MeasuredData>()){}

SegmentData::SegmentData(int id, vector<MeasuredData> values)
	: _id(id), _number_min_metric(NUMBER_MIN_METRIC), _values(values), _blood_values(vector<MeasuredData>())
{
	filterBloodValues();
	prepareInterpolationIst();
	findMaxDate();
	findMinDate();
}

SegmentData::SegmentData(int id, int number_min_metric, vector<MeasuredData> values)
	: _id(id), _number_min_metric(number_min_metric), _values(values), _blood_values(vector<MeasuredData>())
{
	filterBloodValues();
	prepareInterpolationIst();
	findMaxDate();
	findMinDate();
}

void SegmentData::setId(int id){
	this->_id = id;
}
void SegmentData::setNumberMinMetric(int number_min_metric){
	this->_number_min_metric = number_min_metric;
}
void SegmentData::setValues(vector<MeasuredData> values){
	this->_values = values;
	filterBloodValues();
	prepareInterpolationIst();
	findMaxDate();
	findMinDate();
}

void SegmentData::setResult(EquationData result){
	this->_result = result;
}

void SegmentData::filterBloodValues(){
	if (!this->_blood_values.empty()) 
		this->_blood_values.clear();

	if (!this->_values.empty()){
		for (MeasuredData val : this->_values) {
			if (val.getBlood() != 0){
				this->_blood_values.push_back(val);
			}
		}
	}
}

void SegmentData::prepareInterpolationIst(){
	vector<double> date, ist;

	for (MeasuredData val : this->_values) {
		if (val.getIst() != 0) {
			date.push_back(val.getDate());
			ist.push_back(val.getIst());
		}
	}

	_interp_date.setcontent(date.size(), &(date[0]));
	_interp_ist.setcontent(date.size(), &(ist[0]));
	alglib::spline1dbuildakima(_interp_date, _interp_ist, _spline);
	//alglib::spline1dbuildcubic(_interp_date, _interp_ist, _spline);
}

double SegmentData::findMinDate(){
	double minimum = DBL_MAX;
	for (int size = _values.size(), i = 0; i < size; i++){
		minimum = std::min(minimum, _values[i].getDate());
	}
	return _min_date = minimum;
}
double SegmentData::findMaxDate(){
	double maximum = DBL_MIN;
	for (int size = _values.size(), i = 0; i < size; i++){
		maximum = std::max(maximum, _values[i].getDate());
	}
	return _max_date = maximum;
}

std::ostream& operator<<(std::ostream &strm, SegmentData &x) {
	return strm << "SegmentData(" << x.getId() << ", values: " << x.getValues().size() << ", blood values:" << x.getBloodValues().size() << ")";
}