#pragma once
#include <vector>
#include <map>
#include <sstream>
#include <algorithm>
#include "MeasuredData.h"
#include "lib/alglib.net/interpolation.h"
#include "Constants.h"
#include "EquationData.h"

using namespace std;

// Data jednoho segmentu 
class SegmentData
{
public:
	SegmentData(); 
	SegmentData(int id, vector<MeasuredData> values);
	SegmentData(int id, int number_min_metric, vector<MeasuredData> values);

	// Getters 
	int getId() const { return _id; }
	int getNumberMinMetric() const { return _number_min_metric; }
	vector<MeasuredData> getValues() const { return _values; }
	vector<MeasuredData> getBloodValues()  { return _blood_values; }
	double getInterpolationIst(double date) { return alglib::spline1dcalc(_spline, date); }
	EquationData getResult() { return _result; }

	double findMinDate();
	double findMaxDate();

	double getMinDate() const { return _min_date; }
	double getMaxDate() const { return _max_date; }


	// Setters 
	void setId(int id);
	void setNumberMinMetric(int number_min_metric);
	void setValues(vector<MeasuredData> values);
	void setResult(EquationData result);

	void filterBloodValues();
	void prepareInterpolationIst();

private:
	int _id;
	int _number_min_metric;
	vector<MeasuredData> _values;
	vector<MeasuredData> _blood_values;
	EquationData _result;
	double _min_date;
	double _max_date;
	
	alglib::real_1d_array _interp_date, _interp_ist;
	alglib::spline1dinterpolant _spline;
};
std::ostream& operator<<(std::ostream &strm, SegmentData &x);

