#ifndef _SEGMENT_DATA_H
#define _SEGMENT_DATA_H

#include <vector>
#include <map>
#include <sstream>
#include <algorithm>
#include "MeasuredData.h"
#include "lib/alglib.net/interpolation.h"
#include "Constants.h"
#include "EquationData.h"

using namespace std;

//Segment data - contains MeasuredData for segment
class SegmentData
{
public:
	SegmentData(); 
	SegmentData(int id, vector<MeasuredData> values);

	// Getters
	int getId() const { return _id; }
	vector<MeasuredData> getValues() const { return _values; }
	vector<MeasuredData> getBloodValues()  { return _blood_values; }
	EquationData getResult() { return _result; }

	//interpolate Ist value for time when glucose in blood was measured
	double getInterpolationIst(double date) { return alglib::spline1dcalc(_spline, date); }

	double findMinDate();
	double findMaxDate();

	double getMinDate() const { return _min_date; }
	double getMaxDate() const { return _max_date; }

	// Setters 
	void setId(int id);
	void setValues(vector<MeasuredData> values);
	void setResult(EquationData result);

	void filterBloodValues();
	void prepareInterpolationIst();

private:
	//segment ID
	int _id;
	//all segment meadured data
	vector<MeasuredData> _values;
	//measured data which contains blood value
	vector<MeasuredData> _blood_values;
	//parameters with the best metric
	EquationData _result;
	double _min_date;
	double _max_date;
	
	alglib::real_1d_array _interp_date, _interp_ist;
	alglib::spline1dinterpolant _spline;
};
std::ostream& operator<<(std::ostream &strm, SegmentData &x);

#endif