#include "main.h"

using namespace std;
using namespace std::chrono;

int main(int argc, char **argv) {
#ifdef TIME_MEASURING
	cout << "Time measuring ON" << endl;
	high_resolution_clock::time_point full_start, full_end, alg_start, alg_end;
	full_start = high_resolution_clock::now();
#endif
	vector<SegmentData> segments;
	CmdDriver parser(argc, argv);

	std::atexit(exit_handler);

	SqliteDriver sql(parser.getParam("sql"));
	FileDriver file(parser.getParam("bounds"));

	try {
		segments = sql.load(segments);
		file.load();
	}
	catch (string& msg){
		cerr << "Load file problem!\n" << msg << endl;
		return(EXIT_FAILURE);
	}
	
	EquationBounds::printEquationBounds();

#ifdef TIME_MEASURING
	alg_start = high_resolution_clock::now();
#endif
#ifdef _INTEL_TBB_H
	cout << "Run with Intel TTB" << endl;
	int size = segments.size();
	tbb::task_scheduler_init init(4);
	tbb::parallel_for(0, size, [&](int i){
		NelderMead alg(segments[i]);
		cout << "Segment " << std::setfill(' ') << std::setw(3) << segments[i].getId() << " " << segments[i].getResult() << endl;
	});
#else
	cout << "Run without Intel TTB" << endl;
	for (int size = segments.size(), i = 0; i < size; i++) {
		NelderMead alg(segments[i]);
		cout << "Segment " << std::setfill(' ') << std::setw(3) << segments[i].getId() << " " << segments[i].getResult() << endl;
	}
#endif
#ifdef TIME_MEASURING
	alg_end = high_resolution_clock::now();
#endif

	try {
		sql.save(segments);
	}
	catch (string& msg){
		cerr << "Save SQLite problem!\n" << msg << endl;
	}
	
#ifdef TIME_MEASURING
	full_end = high_resolution_clock::now();
	auto full_duration = duration_cast<microseconds>(full_end - full_start).count();
	auto alg_duration = duration_cast<microseconds>(alg_end - alg_start).count();

	cout << "Time execute for Nelder-Mead algorithm: " << alg_duration << "us => " << alg_duration / 1000.0 << "ms => " << alg_duration / 1000000.0 << "s" <<  endl;
	cout << "Time execute for the entire program: " << full_duration << "us => " << alg_duration / 1000.0 << "ms => " << alg_duration / 1000000.0 << "s" << endl;
#endif

	return(EXIT_SUCCESS);
}

void exit_handler(){
	if (DEBUG){
		std::cout << "Press Enter to exit..." << std::endl;
		std::cin.get();
	}
}